import 'package:flutter/material.dart';
import 'package:mighty_news_firebase/components/AppWidgets.dart';
import 'package:mighty_news_firebase/models/NewsData.dart';
import 'package:mighty_news_firebase/utils/Common.dart';
import 'package:mighty_news_firebase/utils/Constants.dart';
import 'package:nb_utils/nb_utils.dart';

class UserNewsItemWidget extends StatelessWidget {
  static String tag = '/UserNewsItemWidget';
  final NewsData? newsData;
  final Function? onTap;

  UserNewsItemWidget({this.newsData, this.onTap});

  @override
  Widget build(BuildContext context) {
    return getIntAsync(POST_LAYOUT, defaultValue: 0) == 0
        ? Container(
            width: context.width(),
            height: 200,
            decoration: boxDecorationRoundedWithShadow(defaultRadius.toInt()),
            child: Stack(
              children: [
                cachedImage(
                  newsData!.image.validate(),
                  fit: BoxFit.cover,
                  width: context.width(),
                  height: context.height(),
                ).cornerRadiusWithClipRRect(defaultRadius),
                Positioned(
                  bottom: 0,
                  right: 0,
                  left: 0,
                  child: Container(
                    width: context.width(),
                    padding: EdgeInsets.all(8),
                    decoration: BoxDecoration(
                      gradient: LinearGradient(
                        colors: [
                          Colors.transparent,
                          Colors.black54,
                          Colors.black,
                        ],
                        stops: [0.0, 0.3, 1.0],
                        begin: Alignment.topCenter,
                        end: Alignment.bottomCenter,
                        tileMode: TileMode.clamp,
                      ),
                    ),
                    child: Column(
                      children: [
                        Text(
                          parseHtmlString(newsData!.title.validate()),
                          style: boldTextStyle(color: Colors.white),
                          maxLines: 3,
                        ),
                        8.height,
                        Row(
                          children: [
                            Icon(Icons.access_time_rounded, size: 18, color: Colors.white),
                            4.width,
                            if (newsData!.updatedAt != null) Text(newsData!.updatedAt!.timeAgo, maxLines: 1, style: secondaryTextStyle(size: 12, color: Colors.white)),
                            4.width,
                            Text('・', style: secondaryTextStyle()),
                            4.width,
                            Text(
                              '${(parseHtmlString(newsData!.content).calculateReadTime()).ceilToDouble().toInt()} ${'min_read'.translate}',
                              style: secondaryTextStyle(size: 12, color: Colors.white),
                            ),
                          ],
                        ),
                      ],
                    ),
                  ),
                ),
              ],
            ).cornerRadiusWithClipRRect(defaultRadius),
          ).onTap(() {
            onTap?.call();
          }, borderRadius: radius()).paddingAll(8)
        : Container(
            margin: EdgeInsets.all(8),
            width: context.width(),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                cachedImage(newsData!.image.validate(), fit: BoxFit.cover, height: 100, width: 140).cornerRadiusWithClipRRect(defaultRadius),
                8.width,
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      parseHtmlString(newsData!.title.validate()),
                      maxLines: 3,
                      style: boldTextStyle(size: 14),
                      overflow: TextOverflow.ellipsis,
                    ),
                    8.height,
                    Row(
                      children: [
                        Icon(Icons.access_time_rounded, size: 18, color: textSecondaryColor),
                        4.width,
                        if (newsData!.updatedAt != null) Text(newsData!.updatedAt!.timeAgo, maxLines: 1, style: secondaryTextStyle(size: 12)),
                        4.width,
                        Text('・', style: secondaryTextStyle()),
                        4.width,
                        Text(
                          '${(parseHtmlString(newsData!.content).calculateReadTime()).ceilToDouble().toInt()} ${'min_read'.translate}',
                          style: secondaryTextStyle(size: 12),
                          overflow: TextOverflow.ellipsis,
                        ).expand(),
                      ],
                    ),
                  ],
                ).expand(),
              ],
            ),
          ).onTap(() {
            onTap?.call();
          }, borderRadius: radius());
  }
}
